@@ -30,6 +30,7 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
 #include <time.h>
 #include <unistd.h>
 
@@ -617,6 +618,28 @@ int get_shm_bytes_per_pixel(uint32_t format)
 	case WL_SHM_FORMAT_ARGB16161616:
 	case WL_SHM_FORMAT_ABGR16161616:
 		return 8;
+	// todo: adjust API to handle bit packed formats
+	case WL_SHM_FORMAT_C1:
+	case WL_SHM_FORMAT_C2:
+	case WL_SHM_FORMAT_C4:
+	case WL_SHM_FORMAT_D1:
+	case WL_SHM_FORMAT_D2:
+	case WL_SHM_FORMAT_D4:
+		goto planar;
+	case WL_SHM_FORMAT_D8:
+		return 1;
+	case WL_SHM_FORMAT_R1:
+	case WL_SHM_FORMAT_R2:
+	case WL_SHM_FORMAT_R4:
+		goto planar;
+	case WL_SHM_FORMAT_R10:
+	case WL_SHM_FORMAT_R12:
+		return 2;
+	case WL_SHM_FORMAT_AVUY8888:
+	case WL_SHM_FORMAT_XVUY8888:
+		return 4;
+	case WL_SHM_FORMAT_P030:
+		goto planar;
 	default:
 		wp_error("Unidentified WL_SHM format %x", format);
 		return -1;
@@ -1560,27 +1583,22 @@ void do_zwp_linux_dmabuf_feedback_v1_evt_done(struct context *ctx)
 	/* Inject messages for filtered tranche parameters here */
 	size_t m = 0;
 	for (size_t i = 0; i < obj->tranche_count; i++) {
-		ctx->message[m] = obj->base.obj_id;
-
-		size_t w = 0;
-		uint16_t *fmts = (uint16_t *)&ctx->message[m + 3];
+		bool empty = true;
 		for (size_t j = 0; j < obj->tranches[i].tranche_size; j++) {
 			uint16_t idx = obj->tranches[i].tranche[j];
 			if (dmabuf_format_permitted(ctx, obj->table[idx].format,
 					    obj->table[idx].modifier)) {
-				fmts[w++] = idx;
+				empty = false;
+				break;
 			}
 		}
-		if (w == 0) {
+
+		if (empty) {
 			/* discard tranche, has no entries */
 			continue;
 		}
 
-		size_t s = 3 + ((w + 1) / 2);
-		ctx->message[m + 1] = message_header_2(
-				4 * (uint32_t)s, 5); // tranche_formats
-		ctx->message[m + 2] = (uint32_t)(2 * w);
-		m += s;
+		size_t s;
 
 		s = 3 + ((sizeof(dev_t) + 3) / 4);
 		ctx->message[m] = obj->base.obj_id;
@@ -1597,6 +1615,22 @@ void do_zwp_linux_dmabuf_feedback_v1_evt_done(struct context *ctx)
 		ctx->message[m + 2] = obj->tranches[i].flags;
 		m += s;
 
+		size_t w = 0;
+		uint16_t *fmts = (uint16_t *)&ctx->message[m + 3];
+		for (size_t j = 0; j < obj->tranches[i].tranche_size; j++) {
+			uint16_t idx = obj->tranches[i].tranche[j];
+			if (dmabuf_format_permitted(ctx, obj->table[idx].format,
+					    obj->table[idx].modifier)) {
+				fmts[w++] = idx;
+			}
+		}
+		s = 3 + ((w + 1) / 2);
+		ctx->message[m] = obj->base.obj_id;
+		ctx->message[m + 1] = message_header_2(
+				4 * (uint32_t)s, 5); // tranche_formats
+		ctx->message[m + 2] = (uint32_t)(2 * w);
+		m += s;
+
 		s = 2;
 		ctx->message[m] = obj->base.obj_id;
 		ctx->message[m + 1] = message_header_2(
@@ -1655,6 +1689,7 @@ void do_zwp_linux_dmabuf_feedback_v1_evt_format_table(
 void do_zwp_linux_dmabuf_feedback_v1_evt_main_device(struct context *ctx,
 		uint32_t device_count, const uint8_t *device_val)
 {
+
 	struct obj_zwp_linux_dmabuf_feedback *obj =
 			(struct obj_zwp_linux_dmabuf_feedback *)ctx->obj;
 	if ((size_t)device_count != sizeof(dev_t)) {
@@ -1662,7 +1697,20 @@ void do_zwp_linux_dmabuf_feedback_v1_evt_main_device(struct context *ctx,
 				(size_t)device_count, sizeof(dev_t));
 		return;
 	}
-	memcpy(&obj->main_device, device_val, sizeof(dev_t));
+
+	if (ctx->on_display_side) {
+		memcpy(&obj->main_device, device_val, sizeof(dev_t));
+	} else {
+		// adopt the main device from the render fd being used
+		struct stat fsdata;
+		memset(&fsdata, 0, sizeof(fsdata));
+		int ret = fstat(ctx->g->render.drm_fd, &fsdata);
+		if (ret == -1) {
+			wp_error("Failed to get render device info");
+			return;
+		}
+		obj->main_device = fsdata.st_rdev;
+	}
 
 	/* todo: add support for changing render devices in waypipe */
 }
@@ -1670,7 +1718,7 @@ void do_zwp_linux_dmabuf_feedback_v1_evt_tranche_done(struct context *ctx)
 {
 	struct obj_zwp_linux_dmabuf_feedback *obj =
 			(struct obj_zwp_linux_dmabuf_feedback *)ctx->obj;
-	if (obj->main_device != obj->current_device) {
+	if (obj->main_device != obj->current_device && ctx->on_display_side) {
 		/* Filter out/ignore all tranches for anything but the main
 		 * device. */
 		return;
