@@ -292,7 +292,7 @@ static int run_recon(const char *control_path, const char *recon_path)
 				control_path, (int)len, 108);
 		return EXIT_FAILURE;
 	}
-	int cfd = open(control_path, O_WRONLY);
+	int cfd = open(control_path, O_WRONLY | O_NOCTTY);
 	if (cfd == -1) {
 		fprintf(stderr, "Failed to open control pipe at \"%s\"\n",
 				control_path);
@@ -430,7 +430,7 @@ static const struct arg_permissions arg_permissions[] = {
 		{ARG_VERSION, MODE_FAIL},
 		{ARG_ALLOW_TILED, MODE_SSH | MODE_CLIENT | MODE_SERVER},
 		{ARG_UNLINK, MODE_SERVER},
-		{ARG_DRMNODE, MODE_SERVER},
+		{ARG_DRMNODE, MODE_SSH | MODE_CLIENT | MODE_SERVER},
 		{ARG_REMOTENODE, MODE_SSH},
 		{ARG_WAYPIPE_BINARY, MODE_SSH},
 		{ARG_LOGIN_SHELL, MODE_SERVER},
